@@ -34,14 +34,14 @@ GST_DEBUG_CATEGORY_STATIC (gst_ebml_write_debug);
 #define GST_CAT_DEFAULT gst_ebml_write_debug
 
 #define _do_init(thing) \
-      GST_DEBUG_CATEGORY_INIT (gst_ebml_write_debug, "ebmlwrite", 0, "Write EBML structured data")
+      GST_DEBUG_CATEGORY_INIT (gst_ebml_write_debug, "GstEbmlWrite", 0, "Write EBML structured data")
 GST_BOILERPLATE_FULL (GstEbmlWrite, gst_ebml_write, GstObject, GST_TYPE_OBJECT,
-    _do_init);
+    _do_init)
 
-static void gst_ebml_write_finalize (GObject * object);
+     static void gst_ebml_write_finalize (GObject * object);
 
-static void
-gst_ebml_write_base_init (gpointer g_class)
+
+     static void gst_ebml_write_base_init (gpointer g_class)
 {
 }
 
@@ -58,13 +58,9 @@ gst_ebml_write_init (GstEbmlWrite * ebml, GstEbmlWriteClass * klass)
 {
   ebml->srcpad = NULL;
   ebml->pos = 0;
-  ebml->last_pos = G_MAXUINT64; /* force newsegment event */
 
   ebml->cache = NULL;
-  ebml->streamheader = NULL;
-  ebml->streamheader_pos = 0;
-  ebml->writing_streamheader = FALSE;
-  ebml->caps = NULL;
+  ebml->cache_size = 0;
 }
 
 static void
@@ -74,21 +70,6 @@ gst_ebml_write_finalize (GObject * object)
 
   gst_object_unref (ebml->srcpad);
 
-  if (ebml->cache) {
-    gst_byte_writer_free (ebml->cache);
-    ebml->cache = NULL;
-  }
-
-  if (ebml->streamheader) {
-    gst_byte_writer_free (ebml->streamheader);
-    ebml->streamheader = NULL;
-  }
-
-  if (ebml->caps) {
-    gst_caps_unref (ebml->caps);
-    ebml->caps = NULL;
-  }
-
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -126,18 +107,12 @@ void
 gst_ebml_write_reset (GstEbmlWrite * ebml)
 {
   ebml->pos = 0;
-  ebml->last_pos = G_MAXUINT64; /* force newsegment event */
 
   if (ebml->cache) {
-    gst_byte_writer_free (ebml->cache);
+    gst_buffer_unref (ebml->cache);
     ebml->cache = NULL;
   }
-
-  if (ebml->caps) {
-    gst_caps_unref (ebml->caps);
-    ebml->caps = NULL;
-  }
-
+  ebml->cache_size = 0;
   ebml->last_write_result = GST_FLOW_OK;
   ebml->timestamp = GST_CLOCK_TIME_NONE;
 }
@@ -161,33 +136,6 @@ gst_ebml_last_write_result (GstEbmlWrite * ebml)
 }
 
 
-void
-gst_ebml_start_streamheader (GstEbmlWrite * ebml)
-{
-  g_return_if_fail (ebml->streamheader == NULL);
-
-  GST_DEBUG ("Starting streamheader at %" G_GUINT64_FORMAT, ebml->pos);
-  ebml->streamheader = gst_byte_writer_new_with_size (1000, FALSE);
-  ebml->streamheader_pos = ebml->pos;
-  ebml->writing_streamheader = TRUE;
-}
-
-GstBuffer *
-gst_ebml_stop_streamheader (GstEbmlWrite * ebml)
-{
-  GstBuffer *buffer;
-
-  if (!ebml->streamheader)
-    return NULL;
-
-  buffer = gst_byte_writer_free_and_get_buffer (ebml->streamheader);
-  ebml->streamheader = NULL;
-  GST_DEBUG ("Streamheader was size %d", GST_BUFFER_SIZE (buffer));
-
-  ebml->writing_streamheader = FALSE;
-  return buffer;
-}
-
 /**
  * gst_ebml_write_set_cache:
  * @ebml: a #GstEbmlWrite.
@@ -203,68 +151,42 @@ gst_ebml_stop_streamheader (GstEbmlWrite * ebml)
 void
 gst_ebml_write_set_cache (GstEbmlWrite * ebml, guint size)
 {
+  /* This is currently broken. I don't know why yet. */
+  return;
+
   g_return_if_fail (ebml->cache == NULL);
 
-  GST_DEBUG ("Starting cache at %" G_GUINT64_FORMAT, ebml->pos);
-  ebml->cache = gst_byte_writer_new_with_size (size, FALSE);
-  ebml->cache_pos = ebml->pos;
-}
-
-static gboolean
-gst_ebml_writer_send_new_segment_event (GstEbmlWrite * ebml, guint64 new_pos)
-{
-  gboolean res;
-
-  GST_INFO ("seeking to %" G_GUINT64_FORMAT, new_pos);
-
-  res = gst_pad_push_event (ebml->srcpad,
-      gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_BYTES, new_pos, -1, 0));
-
-  if (!res)
-    GST_WARNING ("seek to %" G_GUINT64_FORMAT "failed", new_pos);
-
-  return res;
+  ebml->cache = gst_buffer_new_and_alloc (size);
+  ebml->cache_size = size;
+  GST_BUFFER_SIZE (ebml->cache) = 0;
+  GST_BUFFER_OFFSET (ebml->cache) = ebml->pos;
+  ebml->handled = 0;
 }
 
 /**
  * gst_ebml_write_flush_cache:
- * @ebml:      a #GstEbmlWrite.
- * @timestamp: timestamp of the buffer.
+ * @ebml: a #GstEbmlWrite.
  *
  * Flush the cache.
  */
 void
-gst_ebml_write_flush_cache (GstEbmlWrite * ebml, gboolean is_keyframe,
-    GstClockTime timestamp)
+gst_ebml_write_flush_cache (GstEbmlWrite * ebml)
 {
-  GstBuffer *buffer;
-
   if (!ebml->cache)
     return;
 
-  buffer = gst_byte_writer_free_and_get_buffer (ebml->cache);
+  /* this is very important. It may fail, in which case the client
+   * programmer didn't use the cache somewhere. That's fatal. */
+  g_assert (ebml->handled == GST_BUFFER_SIZE (ebml->cache));
+  g_assert (GST_BUFFER_SIZE (ebml->cache) +
+      GST_BUFFER_OFFSET (ebml->cache) == ebml->pos);
+
+  if (ebml->last_write_result == GST_FLOW_OK)
+    ebml->last_write_result = gst_pad_push (ebml->srcpad, ebml->cache);
+
   ebml->cache = NULL;
-  GST_DEBUG ("Flushing cache of size %d", GST_BUFFER_SIZE (buffer));
-  gst_buffer_set_caps (buffer, ebml->caps);
-  GST_BUFFER_TIMESTAMP (buffer) = timestamp;
-  GST_BUFFER_OFFSET (buffer) = ebml->pos - GST_BUFFER_SIZE (buffer);
-  GST_BUFFER_OFFSET_END (buffer) = ebml->pos;
-  if (ebml->last_write_result == GST_FLOW_OK) {
-    if (GST_BUFFER_OFFSET (buffer) != ebml->last_pos) {
-      gst_ebml_writer_send_new_segment_event (ebml, GST_BUFFER_OFFSET (buffer));
-      GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
-    }
-    if (ebml->writing_streamheader) {
-      GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_IN_CAPS);
-    }
-    if (!is_keyframe) {
-      GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
-    }
-    ebml->last_pos = ebml->pos;
-    ebml->last_write_result = gst_pad_push (ebml->srcpad, buffer);
-  } else {
-    gst_buffer_unref (buffer);
-  }
+  ebml->cache_size = 0;
+  ebml->handled = 0;
 }
 
 
@@ -279,7 +201,7 @@ gst_ebml_write_flush_cache (GstEbmlWrite * ebml, gboolean is_keyframe,
  * Returns: A new #GstBuffer.
  */
 static GstBuffer *
-gst_ebml_write_element_new (GstEbmlWrite * ebml, guint8 ** data_out, guint size)
+gst_ebml_write_element_new (GstEbmlWrite * ebml, guint size)
 {
   /* Create new buffer of size + ID + length */
   GstBuffer *buf;
@@ -287,10 +209,20 @@ gst_ebml_write_element_new (GstEbmlWrite * ebml, guint8 ** data_out, guint size)
   /* length, ID */
   size += 12;
 
-  buf = gst_buffer_new_and_alloc (size);
-  GST_BUFFER_TIMESTAMP (buf) = ebml->timestamp;
+  /* prefer cache */
+  if (ebml->cache) {
+    if (ebml->cache_size - GST_BUFFER_SIZE (ebml->cache) < size) {
+      GST_LOG ("Cache available, but too small. Clearing...");
+      gst_ebml_write_flush_cache (ebml);
+    } else {
+      return ebml->cache;
+    }
+  }
 
-  *data_out = GST_BUFFER_DATA (buf);
+  /* else, use a one-element buffer. This is slower */
+  buf = gst_buffer_new_and_alloc (size);
+  GST_BUFFER_SIZE (buf) = 0;
+  GST_BUFFER_TIMESTAMP (buf) = ebml->timestamp;
 
   return buf;
 }
@@ -298,15 +230,15 @@ gst_ebml_write_element_new (GstEbmlWrite * ebml, guint8 ** data_out, guint size)
 
 /**
  * gst_ebml_write_element_id:
- * @data_inout: Pointer to data pointer
+ * @buf: Buffer to which id should be written.
  * @id: Element ID that should be written.
  * 
  * Write element ID into a buffer.
  */
 static void
-gst_ebml_write_element_id (guint8 ** data_inout, guint32 id)
+gst_ebml_write_element_id (GstBuffer * buf, guint32 id)
 {
-  guint8 *data = *data_inout;
+  guint8 *data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
   guint bytes = 4, mask = 0x10;
 
   /* get ID length */
@@ -323,7 +255,7 @@ gst_ebml_write_element_id (guint8 ** data_inout, guint32 id)
   }
 
   /* write out, BE */
-  *data_inout += bytes;
+  GST_BUFFER_SIZE (buf) += bytes;
   while (bytes--) {
     data[bytes] = id & 0xff;
     id >>= 8;
@@ -333,15 +265,15 @@ gst_ebml_write_element_id (guint8 ** data_inout, guint32 id)
 
 /**
  * gst_ebml_write_element_size:
- * @data_inout: Pointer to data pointer
+ * @buf: #GstBuffer to which size should be written.
  * @size: Element length.
- *
+ * 
  * Write element length into a buffer.
  */
 static void
-gst_ebml_write_element_size (guint8 ** data_inout, guint64 size)
+gst_ebml_write_element_size (GstBuffer * buf, guint64 size)
 {
-  guint8 *data = *data_inout;
+  guint8 *data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
   guint bytes = 1, mask = 0x80;
 
   if (size != GST_EBML_SIZE_UNKNOWN) {
@@ -365,7 +297,7 @@ gst_ebml_write_element_size (guint8 ** data_inout, guint64 size)
   }
 
   /* write out, BE, with length size marker */
-  *data_inout += bytes;
+  GST_BUFFER_SIZE (buf) += bytes;
   while (bytes-- > 0) {
     data[bytes] = size & 0xff;
     size >>= 8;
@@ -377,18 +309,19 @@ gst_ebml_write_element_size (guint8 ** data_inout, guint64 size)
 
 /**
  * gst_ebml_write_element_data:
- * @data_inout: Pointer to data pointer
+ * @buf: #GstBuffer to which data should be written.
  * @write: Data that should be written.
  * @length: Length of the data.
  *
  * Write element data into a buffer.
  */
 static void
-gst_ebml_write_element_data (guint8 ** data_inout, guint8 * write,
-    guint64 length)
+gst_ebml_write_element_data (GstBuffer * buf, guint8 * write, guint64 length)
 {
-  memcpy (*data_inout, write, length);
-  *data_inout += length;
+  guint8 *data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
+
+  memcpy (data, write, length);
+  GST_BUFFER_SIZE (buf) += length;
 }
 
 
@@ -396,59 +329,27 @@ gst_ebml_write_element_data (guint8 ** data_inout, guint8 * write,
  * gst_ebml_write_element_push:
  * @ebml: #GstEbmlWrite
  * @buf: #GstBuffer to be written.
- * @buf_data: Start of data to push from @buf (or NULL for whole buffer).
- * @buf_data_end: Data pointer positioned after the last byte in @buf_data (or
- * NULL for whole buffer).
  * 
  * Write out buffer by moving it to the next element.
  */
 static void
-gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf,
-    guint8 * buf_data, guint8 * buf_data_end)
+gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf)
 {
-  guint data_size;
-
-  if (!buf_data)
-    buf_data = GST_BUFFER_DATA (buf);
-
-  if (buf_data_end) {
-    data_size = buf_data_end - buf_data;
-    GST_BUFFER_SIZE (buf) = data_size;
-  } else {
-    data_size = GST_BUFFER_SIZE (buf);
-  }
+  guint data_size = GST_BUFFER_SIZE (buf) - ebml->handled;
 
   ebml->pos += data_size;
+  if (buf == ebml->cache) {
+    ebml->handled += data_size;
+  }
 
   /* if there's no cache, then don't push it! */
-  if (ebml->writing_streamheader) {
-    gst_byte_writer_put_data (ebml->streamheader, buf_data, data_size);
-  }
   if (ebml->cache) {
-    gst_byte_writer_put_data (ebml->cache, buf_data, data_size);
-    gst_buffer_unref (buf);
+    g_assert (buf == ebml->cache);
     return;
   }
 
-  if (ebml->last_write_result == GST_FLOW_OK) {
-    buf = gst_buffer_make_metadata_writable (buf);
-    gst_buffer_set_caps (buf, ebml->caps);
-    GST_BUFFER_OFFSET (buf) = ebml->pos - data_size;
-    GST_BUFFER_OFFSET_END (buf) = ebml->pos;
-    if (ebml->writing_streamheader) {
-      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
-    }
-    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT);
-
-    if (GST_BUFFER_OFFSET (buf) != ebml->last_pos) {
-      gst_ebml_writer_send_new_segment_event (ebml, GST_BUFFER_OFFSET (buf));
-      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
-    }
-    ebml->last_pos = ebml->pos;
+  if (ebml->last_write_result == GST_FLOW_OK)
     ebml->last_write_result = gst_pad_push (ebml->srcpad, buf);
-  } else {
-    gst_buffer_unref (buf);
-  }
 }
 
 
@@ -462,36 +363,35 @@ gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf,
 void
 gst_ebml_write_seek (GstEbmlWrite * ebml, guint64 pos)
 {
-  if (ebml->writing_streamheader) {
-    GST_DEBUG ("wanting to seek to pos %" G_GUINT64_FORMAT, pos);
-    if (pos >= ebml->streamheader_pos &&
-        pos <= ebml->streamheader_pos + ebml->streamheader->parent.size) {
-      gst_byte_writer_set_pos (ebml->streamheader,
-          pos - ebml->streamheader_pos);
-      GST_DEBUG ("seeked in streamheader to position %" G_GUINT64_FORMAT,
-          pos - ebml->streamheader_pos);
-    } else {
-      GST_WARNING
-          ("we are writing streamheader still and seek is out of bounds");
-    }
-  }
+  GstEvent *seek;
+  GstPad *peer_pad;
+
   /* Cache seeking. A bit dangerous, we assume the client writer
    * knows what he's doing... */
   if (ebml->cache) {
     /* within bounds? */
-    if (pos >= ebml->cache_pos &&
-        pos <= ebml->cache_pos + ebml->cache->parent.size) {
-      GST_DEBUG ("seeking in cache to %" G_GUINT64_FORMAT, pos);
+    if (pos >= GST_BUFFER_OFFSET (ebml->cache) &&
+        pos < GST_BUFFER_OFFSET (ebml->cache) + ebml->cache_size) {
+      GST_BUFFER_SIZE (ebml->cache) = pos - GST_BUFFER_OFFSET (ebml->cache);
+      if (ebml->pos > pos)
+        ebml->handled -= ebml->pos - pos;
+      else
+        ebml->handled += pos - ebml->pos;
       ebml->pos = pos;
-      gst_byte_writer_set_pos (ebml->cache, ebml->pos - ebml->cache_pos);
-      return;
     } else {
       GST_LOG ("Seek outside cache range. Clearing...");
-      gst_ebml_write_flush_cache (ebml, FALSE, GST_CLOCK_TIME_NONE);
+      gst_ebml_write_flush_cache (ebml);
     }
   }
 
-  GST_INFO ("scheduling seek to %" G_GUINT64_FORMAT, pos);
+  seek = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_BYTES, pos, -1, 0);
+  peer_pad = GST_PAD_PEER (ebml->srcpad);
+  if (peer_pad) {
+    gst_pad_send_event (peer_pad, seek);
+  } else {
+    GST_WARNING_OBJECT (ebml, "Can not seek: no peer pad");
+  }
+
   ebml->pos = pos;
 }
 
@@ -520,19 +420,19 @@ gst_ebml_write_get_uint_size (guint64 num)
 
 /**
  * gst_ebml_write_set_uint:
- * @data_inout: Pointer to data pointer
+ * @buf: #GstBuffer to which ithe number should be written.
  * @num: Number to be written.
  * @size: Encoded number length.
  *
  * Write an uint into a buffer.
  */
 static void
-gst_ebml_write_set_uint (guint8 ** data_inout, guint64 num, guint size)
+gst_ebml_write_set_uint (GstBuffer * buf, guint64 num, guint size)
 {
-  guint8 *data = *data_inout;
-
-  *data_inout += size;
+  guint8 *data;
 
+  data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
+  GST_BUFFER_SIZE (buf) += size;
   while (size-- > 0) {
     data[size] = num & 0xff;
     num >>= 8;
@@ -551,18 +451,14 @@ gst_ebml_write_set_uint (guint8 ** data_inout, guint64 num, guint size)
 void
 gst_ebml_write_uint (GstEbmlWrite * ebml, guint32 id, guint64 num)
 {
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
   guint size = gst_ebml_write_get_uint_size (num);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, sizeof (num));
-  data_end = data_start;
-
   /* write */
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, size);
-  gst_ebml_write_set_uint (&data_end, num, size);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, size);
+  gst_ebml_write_set_uint (buf, num, size);
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
@@ -577,8 +473,8 @@ gst_ebml_write_uint (GstEbmlWrite * ebml, guint32 id, guint64 num)
 void
 gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
 {
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
+
   /* if the signed number is on the edge of a extra-byte,
    * then we'll fall over when detecting it. Example: if I
    * have a number (-)0x8000 (G_MINSHORT), then my abs()<<1
@@ -586,9 +482,6 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
   guint64 unum = (num < 0 ? (-num - 1) << 1 : num << 1);
   guint size = gst_ebml_write_get_uint_size (unum);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, sizeof (num));
-  data_end = data_start;
-
   /* make unsigned */
   if (num >= 0) {
     unum = num;
@@ -599,10 +492,10 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
   }
 
   /* write */
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, size);
-  gst_ebml_write_set_uint (&data_end, unum, size);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, size);
+  gst_ebml_write_set_uint (buf, unum, size);
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
@@ -617,17 +510,22 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
 void
 gst_ebml_write_float (GstEbmlWrite * ebml, guint32 id, gdouble num)
 {
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+  gint n;
+#endif
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, sizeof (num));
-  data_end = data_start;
-
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, 8);
-  num = GDOUBLE_TO_BE (num);
-  gst_ebml_write_element_data (&data_end, (guint8 *) & num, 8);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, 8);
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+  for (n = 0; n < 8; n++) {
+    GST_BUFFER_DATA (buf)[GST_BUFFER_SIZE (buf)] = ((guint8 *) & num)[7 - n];
+    GST_BUFFER_SIZE (buf) += 1;
+  }
+#else
+  gst_ebml_write_element_data (buf, (guint8 *) & num, 8);
+#endif
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
@@ -643,16 +541,12 @@ void
 gst_ebml_write_ascii (GstEbmlWrite * ebml, guint32 id, const gchar * str)
 {
   gint len = strlen (str) + 1;  /* add trailing '\0' */
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, len);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, len);
-  data_end = data_start;
-
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, len);
-  gst_ebml_write_element_data (&data_end, (guint8 *) str, len);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, len);
+  gst_ebml_write_element_data (buf, (guint8 *) str, len);
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
@@ -701,52 +595,43 @@ gst_ebml_write_date (GstEbmlWrite * ebml, guint32 id, gint64 date)
 guint64
 gst_ebml_write_master_start (GstEbmlWrite * ebml, guint32 id)
 {
-  guint64 pos = ebml->pos;
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  guint64 pos = ebml->pos, t;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, 0);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, 0);
-  data_end = data_start;
-
-  gst_ebml_write_element_id (&data_end, id);
-  pos += data_end - data_start;
-  gst_ebml_write_element_size (&data_end, GST_EBML_SIZE_UNKNOWN);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  t = GST_BUFFER_SIZE (buf);
+  gst_ebml_write_element_id (buf, id);
+  pos += GST_BUFFER_SIZE (buf) - t;
+  gst_ebml_write_element_size (buf, GST_EBML_SIZE_UNKNOWN);
+  gst_ebml_write_element_push (ebml, buf);
 
   return pos;
 }
 
 
 /**
- * gst_ebml_write_master_finish_full:
+ * gst_ebml_write_master_finish:
  * @ebml: #GstEbmlWrite
  * @startpos: Master starting position.
  *
- * Finish writing master element.  Size of master element is difference between
- * current position and the element start, and @extra_size added to this.
+ * Finish writing master element.
  */
 void
-gst_ebml_write_master_finish_full (GstEbmlWrite * ebml, guint64 startpos,
-    guint64 extra_size)
+gst_ebml_write_master_finish (GstEbmlWrite * ebml, guint64 startpos)
 {
   guint64 pos = ebml->pos;
   GstBuffer *buf;
 
   gst_ebml_write_seek (ebml, startpos);
-
-  buf = gst_buffer_new_and_alloc (8);
-  GST_WRITE_UINT64_BE (GST_BUFFER_DATA (buf),
-      (G_GINT64_CONSTANT (1) << 56) | (pos - startpos - 8 + extra_size));
-
-  gst_ebml_write_element_push (ebml, buf, NULL, NULL);
+  buf = gst_ebml_write_element_new (ebml, 0);
+  startpos =
+      GUINT64_TO_BE ((G_GINT64_CONSTANT (1) << 56) | (pos - startpos - 8));
+  memcpy (GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf), (guint8 *) & startpos,
+      8);
+  GST_BUFFER_SIZE (buf) += 8;
+  gst_ebml_write_element_push (ebml, buf);
   gst_ebml_write_seek (ebml, pos);
 }
 
-void
-gst_ebml_write_master_finish (GstEbmlWrite * ebml, guint64 startpos)
-{
-  gst_ebml_write_master_finish_full (ebml, startpos, 0);
-}
 
 /**
  * gst_ebml_write_binary:
@@ -761,16 +646,12 @@ void
 gst_ebml_write_binary (GstEbmlWrite * ebml,
     guint32 id, guint8 * binary, guint64 length)
 {
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, length);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, length);
-  data_end = data_start;
-
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, length);
-  gst_ebml_write_element_data (&data_end, binary, length);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, length);
+  gst_ebml_write_element_data (buf, binary, length);
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
@@ -790,29 +671,25 @@ gst_ebml_write_binary (GstEbmlWrite * ebml,
 void
 gst_ebml_write_buffer_header (GstEbmlWrite * ebml, guint32 id, guint64 length)
 {
-  GstBuffer *buf;
-  guint8 *data_start, *data_end;
+  GstBuffer *buf = gst_ebml_write_element_new (ebml, 0);
 
-  buf = gst_ebml_write_element_new (ebml, &data_start, 0);
-  data_end = data_start;
-
-  gst_ebml_write_element_id (&data_end, id);
-  gst_ebml_write_element_size (&data_end, length);
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  gst_ebml_write_element_id (buf, id);
+  gst_ebml_write_element_size (buf, length);
+  gst_ebml_write_element_push (ebml, buf);
 }
 
 
 /**
  * gst_ebml_write_buffer:
  * @ebml: #GstEbmlWrite
- * @buf: #GstBuffer cointaining the data.
+ * @data: #GstBuffer cointaining the data.
  *
  * Write  binary element (see gst_ebml_write_buffer_header).
  */
 void
-gst_ebml_write_buffer (GstEbmlWrite * ebml, GstBuffer * buf)
+gst_ebml_write_buffer (GstEbmlWrite * ebml, GstBuffer * data)
 {
-  gst_ebml_write_element_push (ebml, buf, NULL, NULL);
+  gst_ebml_write_element_push (ebml, data);
 }
 
 
@@ -836,15 +713,11 @@ gst_ebml_replace_uint (GstEbmlWrite * ebml, guint64 pos, guint64 num)
 {
   guint64 oldpos = ebml->pos;
   GstBuffer *buf = gst_buffer_new_and_alloc (8);
-  guint8 *data_start, *data_end;
-
-  data_start = GST_BUFFER_DATA (buf);
-  data_end = data_start;
 
   gst_ebml_write_seek (ebml, pos);
-  gst_ebml_write_set_uint (&data_end, num, 8);
-
-  gst_ebml_write_element_push (ebml, buf, data_start, data_end);
+  GST_BUFFER_SIZE (buf) = 0;
+  gst_ebml_write_set_uint (buf, num, 8);
+  gst_ebml_write_element_push (ebml, buf);
   gst_ebml_write_seek (ebml, oldpos);
 }
 
@@ -857,8 +730,7 @@ gst_ebml_replace_uint (GstEbmlWrite * ebml, guint64 pos, guint64 num)
  * Write EBML header.
  */
 void
-gst_ebml_write_header (GstEbmlWrite * ebml, const gchar * doctype,
-    guint version)
+gst_ebml_write_header (GstEbmlWrite * ebml, gchar * doctype, guint version)
 {
   guint64 pos;
 
@@ -878,5 +750,5 @@ gst_ebml_write_header (GstEbmlWrite * ebml, const gchar * doctype,
   gst_ebml_write_uint (ebml, GST_EBML_ID_DOCTYPEVERSION, version);
   gst_ebml_write_uint (ebml, GST_EBML_ID_DOCTYPEREADVERSION, version);
   gst_ebml_write_master_finish (ebml, pos);
-  gst_ebml_write_flush_cache (ebml, FALSE, 0);
+  gst_ebml_write_flush_cache (ebml);
 }
